@@ -1,7 +1,21 @@
 package Image::Magick;
 
-# Released Feb. 17, 1997  by Kyle Shorter (magick@wizards.dupont.com)
-# Public Domain
+#  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization
+#  dedicated to making software imaging solutions freely available.
+#
+#  You may not use this file except in compliance with the License.  You may
+#  obtain a copy of the License at
+#
+#    https://imagemagick.org/script/license.php
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
+#
+#  Initial version, written by Kyle Shorter.
+
 
 use strict;
 use Carp;
@@ -30,12 +44,13 @@ require AutoLoader;
       ConfigureError FatalErrorException
     );
 
-$VERSION = '6.4.8';
+$VERSION = '7.0.10';
 
 sub AUTOLOAD {
     # This AUTOLOAD is used to 'autoload' constants from the constant()
     # XS function.  If a constant is not found then control is passed
     # to the AUTOLOAD in AutoLoader.
+    no warnings;
 
     my $constname;
     ($constname = $AUTOLOAD) =~ s/.*:://;
@@ -88,7 +103,7 @@ __END__
 
 =head1 NAME
 
-Image::Magick - Perl extension for calling ImageMagick's libMagick methods
+Image::Magick - objected-oriented Perl interface to ImageMagick. Use it to read, manipulate, or write an image or image sequence from within a Perl script.
 
 =head1 SYNOPSIS
 
@@ -107,10 +122,14 @@ This Perl extension allows the reading, manipulation and writing of
 a large number of image file formats using the ImageMagick library.
 It was originally developed to be used by CGI scripts for Web pages.
 
-A Web page has been set up for this extension. See:
+A web page has been set up for this extension. See:
 
-	 file:///usr/share/doc/ImageMagick-6.4.8/www/perl-magick.html
-	 http://www.imagemagick.org/script/perl-magick.php
+	 file:///usr/local/share/doc/ImageMagick-7.0.0/www/perl-magick.html
+	 https://imagemagick.org/script/perl-magick.php
+
+If you have problems, go to
+
+   https://imagemagick.org/discourse-server/viewforum.php?f=7
 
 =head1 AUTHOR
 
